@@ -13,7 +13,7 @@ except ImportError:
 
 def have_gdal():
     try:
-        import gdal
+        from osgeo import gdal
     except ImportError:
         return False
     else:
@@ -22,7 +22,7 @@ def have_gdal():
 
 def _fix_conda_gdal_hack():
     import os
-    import gdal  # NOQA
+    from osgeo import gdal  # NOQA
     if os.pathsep != ';' and ';' in os.environ["PATH"]:
         # Workaround conda gdal issue
         # https://github.com/OSGeo/gdal/issues/1231
@@ -49,7 +49,7 @@ def _doctest_check_cog(data, fpath):
 
 
 def _numpy_to_gdal_dtype(numpy_dtype):
-    import gdal
+    from osgeo import gdal
     kindsize = (numpy_dtype.kind, numpy_dtype.itemsize)
     if kindsize == ('u', 1):
         eType = gdal.GDT_Byte
@@ -169,7 +169,7 @@ def _imwrite_cloud_optimized_geotiff(fpath, data, compress='auto',
         xdoctest -m ndsampler.utils.util_gdal _imwrite_cloud_optimized_geotiff
 
     Example:
-        >>> # xdoctest: +REQUIRES(module:gdal)
+        >>> # xdoctest: +REQUIRES(module:osgeo)
         >>> from ndsampler.utils.util_gdal import *  # NOQA
         >>> from ndsampler.utils.util_gdal import _imwrite_cloud_optimized_geotiff
         >>> from ndsampler.utils.util_gdal import _doctest_check_cog
@@ -212,7 +212,7 @@ def _imwrite_cloud_optimized_geotiff(fpath, data, compress='auto',
         >>>     with timer:
         >>>         _imwrite_cloud_optimized_geotiff(fpath2, data)
     """
-    import gdal
+    from osgeo import gdal
     if len(data.shape) == 2:
         data = data[:, :, None]
 
@@ -387,7 +387,7 @@ def _cli_convert_cloud_optimized_geotiff(src_fpath, dst_fpath, compress='auto',
         >>> for timer in ti.reset('SSD-api'):
         >>>     _cli_convert_cloud_optimized_geotiff(src_fpath, dst_fpath)
     """
-    import gdal
+    from osgeo import gdal
     options = [
         'TILED=YES',
         'BIGTIFF=YES',
@@ -454,7 +454,7 @@ def _api_convert_cloud_optimized_geotiff(src_fpath, dst_fpath,
         xdoctest -m ~/code/ndsampler/ndsampler/utils/util_gdal.py _api_convert_cloud_optimized_geotiff --bench
 
     """
-    import gdal
+    from osgeo import gdal
     # data_set = gdal.Open(src_fpath)
     data_set = gdal.Open(src_fpath, gdal.GA_ReadOnly)
 
@@ -506,7 +506,7 @@ def _api_convert_cloud_optimized_geotiff2(src_fpath, dst_fpath,
     CommandLine:
         xdoctest -m ~/code/ndsampler/ndsampler/utils/util_gdal.py _api_convert_cloud_optimized_geotiff --bench
     """
-    import gdal
+    from osgeo import gdal
     # data_set = gdal.Open(src_fpath)
     data_set = gdal.Open(src_fpath, gdal.GA_ReadOnly)
 
@@ -541,7 +541,7 @@ def _api_convert_cloud_optimized_geotiff2(src_fpath, dst_fpath,
 
     translate_opts = gdal.TranslateOptions(format='GTiff', creationOptions=options)
     data_set2 = gdal.Translate(dst_fpath, data_set, options=translate_opts)
-    data_set2 = None
+    data_set2 = None  # NOQA
 
     # # Copy the in-memory dataset to an on-disk GeoTiff
     # driver2 = gdal.GetDriverByName(str('GTiff'))
@@ -572,7 +572,7 @@ class LazyGDalFrameFile(ub.NiceRepr):
               height, width, and channels if possible.
 
     Example:
-        >>> # xdoctest: +REQUIRES(module:gdal)
+        >>> # xdoctest: +REQUIRES(module:osgeo)
         >>> from ndsampler.utils.util_gdal import *  # NOQA
         >>> self = LazyGDalFrameFile.demo()
         >>> cog_fpath = self.cog_fpath
@@ -587,20 +587,35 @@ class LazyGDalFrameFile(ub.NiceRepr):
     """
     def __init__(self, cog_fpath):
         self.cog_fpath = cog_fpath
+        self.fpath = cog_fpath
 
     @ub.memoize_property
     def _ds(self):
-        import gdal
+        from osgeo import gdal
         if not exists(self.cog_fpath):
             raise Exception('File does not exist: {}'.format(self.cog_fpath))
         ds = gdal.Open(self.cog_fpath, gdal.GA_ReadOnly)
         return ds
 
     @classmethod
-    def demo(cls):
-        from ndsampler.abstract_frames import SimpleFrames
-        self = SimpleFrames.demo()
-        self = self._load_image_cog(1)
+    def demo(cls, key='astro', dsize=None):
+        """
+        Ignore:
+            >>> from ndsampler.utils.util_gdal import *  # NOQA
+            >>> self = LazyGDalFrameFile.demo(dsize=(6600, 4400))
+        """
+        cache_dpath = ub.ensure_app_cache_dir('ndsampler/demo')
+        fpath = join(cache_dpath, key + '.cog.tiff')
+        depends = ub.odict(dsize=dsize)
+        cfgstr = ub.hash_data(depends)
+        stamp = ub.CacheStamp(fname=key, cfgstr=cfgstr, dpath=cache_dpath,
+                              product=[fpath])
+        if stamp.expired():
+            import kwimage
+            img = kwimage.grab_test_image(key, dsize=dsize)
+            kwimage.imwrite(fpath, img, backend='gdal')
+            stamp.renew()
+        self = cls(fpath)
         return self
 
     @property
@@ -629,6 +644,16 @@ class LazyGDalFrameFile(ub.NiceRepr):
         """
         References:
             https://gis.stackexchange.com/questions/162095/gdal-driver-create-typeerror
+
+        Ignore:
+            >>> from ndsampler.utils.util_gdal import *  # NOQA
+            >>> self = LazyGDalFrameFile.demo(dsize=(6600, 4400))
+            >>> index = [slice(2100, 2508, None), slice(4916, 5324, None), None]
+            >>> img_part = self[index]
+            >>> # xdoctest: +REQUIRES(--show)
+            >>> import kwplot
+            >>> kwplot.autompl()
+            >>> kwplot.imshow(img_part)
         """
         ds = self._ds
         width = ds.RasterXSize
@@ -655,26 +680,45 @@ class LazyGDalFrameFile(ub.NiceRepr):
             rb_indices = range(C)
             assert len(trailing_part) <= 1
 
-        # TODO: preallocate like kwimage
-        channels = []
-        for i in rb_indices:
-            rb = ds.GetRasterBand(1 + i)
-            xsize = rb.XSize
-            ysize = rb.YSize
+        ystart, ystop = map(int, [ypart.start, ypart.stop])
+        xstart, xstop = map(int, [xpart.start, xpart.stop])
 
-            ystart, ystop = map(int, [ypart.start, ypart.stop])
-            ysize = ystop - ystart
+        ysize = ystop - ystart
+        xsize = xstop - xstart
 
-            xstart, xstop = map(int, [xpart.start, xpart.stop])
-            xsize = xstop - xstart
+        gdalkw = dict(xoff=xstart, yoff=ystart,
+                      win_xsize=xsize, win_ysize=ysize)
 
-            gdalkw = dict(xoff=xstart, yoff=ystart, win_xsize=xsize, win_ysize=ysize)
-            channel = rb.ReadAsArray(**gdalkw)
-            channels.append(channel)
-
-        img_part = np.dstack(channels)
+        PREALLOC = 1
+        if PREALLOC:
+            # preallocate like kwimage.im_io._imread_gdal
+            from kwimage.im_io import _gdal_to_numpy_dtype
+            shape = (ysize, xsize, len(rb_indices))
+            bands = [ds.GetRasterBand(1 + rb_idx)
+                     for rb_idx in rb_indices]
+            gdal_dtype = bands[0].DataType
+            dtype = _gdal_to_numpy_dtype(gdal_dtype)
+            img_part = np.empty(shape, dtype=dtype)
+            for out_idx, rb in enumerate(bands):
+                img_part[:, :, out_idx] = rb.ReadAsArray(**gdalkw)
+        else:
+            channels = []
+            for rb_idx in rb_indices:
+                rb = ds.GetRasterBand(1 + rb_idx)
+                channel = rb.ReadAsArray(**gdalkw)
+                channels.append(channel)
+            img_part = np.dstack(channels)
         return img_part
 
+    def __array__(self):
+        """
+        Allow this object to be passed to np.asarray
+
+        References:
+            https://numpy.org/doc/stable/user/basics.dispatch.html
+        """
+        return self[:]
+
     def validate(self, orig_fpath=None, orig_data=None):
         """
         Check for any corruption issues
@@ -757,7 +801,7 @@ def validate_nonzero_data(file):
     May fail on all-black images
 
     Example:
-        >>> # xdoctest: +REQUIRES(module:gdal)
+        >>> # xdoctest: +REQUIRES(module:osgeo)
         >>> from ndsampler.utils.util_gdal import LazyGDalFrameFile
         >>> import kwimage
         >>> gpath = kwimage.grab_test_image_fpath()
